# Languages.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from UserDict import *
import os.path
import string

from GTK import *
import gnome.config

import GTKSCINTILLA
import Moleskine

_CONFIG_FILE = '/Moleskine/LanguageModes/'

Lexers = {
    'None'          : GTKSCINTILLA.LEXER_NULL,
    'Python'        : GTKSCINTILLA.LEXER_PYTHON,
    'C/C++'         : GTKSCINTILLA.LEXER_CPP,
    'HTML'          : GTKSCINTILLA.LEXER_HTML,
    'XML'           : GTKSCINTILLA.LEXER_XML,
    'Perl'          : GTKSCINTILLA.LEXER_PERL,
    'SQL'           : GTKSCINTILLA.LEXER_SQL,
    'Visual Basic'  : GTKSCINTILLA.LEXER_VB,
    'Properties'    : GTKSCINTILLA.LEXER_PROPERTIES,
    'Error list'    : GTKSCINTILLA.LEXER_ERRORLIST,
    'Makefile'      : GTKSCINTILLA.LEXER_MAKEFILE,
    'Batch file'    : GTKSCINTILLA.LEXER_BATCH,
    'LaTeX'         : GTKSCINTILLA.LEXER_LATEX,
    'LUA'           : GTKSCINTILLA.LEXER_LUA,
    'Diff'          : GTKSCINTILLA.LEXER_DIFF,
    'Apache Config' : GTKSCINTILLA.LEXER_CONF,
    'Pascal'        : GTKSCINTILLA.LEXER_PASCAL,
    'Avenue'        : GTKSCINTILLA.LEXER_AVE,
    'Ada'           : GTKSCINTILLA.LEXER_ADA,
    'Lisp'          : GTKSCINTILLA.LEXER_LISP,
    'Ruby'          : GTKSCINTILLA.LEXER_RUBY,
    'Eiffel'        : GTKSCINTILLA.LEXER_EIFFEL,
    'Eiffel KW'     : GTKSCINTILLA.LEXER_EIFFELKW,
    'Tcl'           : GTKSCINTILLA.LEXER_TCL,
    'NN Crontab'    : GTKSCINTILLA.LEXER_NNCRONTAB,
    'Bullant'       : GTKSCINTILLA.LEXER_BULLANT,
    'VB Script'     : GTKSCINTILLA.LEXER_VBSCRIPT,
    'ASP'           : GTKSCINTILLA.LEXER_ASP,
    'PHP'           : GTKSCINTILLA.LEXER_PHP,
    'Baan'          : GTKSCINTILLA.LEXER_BAAN,
}

Styles = {
    GTKSCINTILLA.LEXER_PYTHON : {
        'White space'               : GTKSCINTILLA.PYTHON_WHITE_SPACE,
        'Comment line'              : GTKSCINTILLA.PYTHON_COMMENT_LINE,
        'Number'                    : GTKSCINTILLA.PYTHON_NUMBER,
        'String double quote'       : GTKSCINTILLA.PYTHON_STRING_DOUBLE,
        'String single quote'       : GTKSCINTILLA.PYTHON_STRING_SINGLE,
        'Keyword'                   : GTKSCINTILLA.PYTHON_KEYWORD,
        'Long string double quote'  : GTKSCINTILLA.PYTHON_TRIPLE_DOUBLE,
        'Long string single quote'  : GTKSCINTILLA.PYTHON_TRIPLE_SINGLE,
        'Class name definition'     : GTKSCINTILLA.PYTHON_CLASS_STATEMENT,
        'Function name definition'  : GTKSCINTILLA.PYTHON_DEF_STATEMENT,
        'Operator'                  : GTKSCINTILLA.PYTHON_OPERATOR,
        'Identifier'                : GTKSCINTILLA.PYTHON_IDENTIFIER,
        'Comment block'             : GTKSCINTILLA.PYTHON_COMMENT_BLOCK,
        'String EOL'                : GTKSCINTILLA.PYTHON_STRING_EOL },
    GTKSCINTILLA.LEXER_CPP : {
        'White space'           : GTKSCINTILLA.CPP_WHITE_SPACE,
        'Comment block'         : GTKSCINTILLA.CPP_COMMENT_BLOCK,
        'Comment line'          : GTKSCINTILLA.CPP_COMMENT_LINE,
        'Comment doc'           : GTKSCINTILLA.CPP_COMMENT_DOC,
        'Number'                : GTKSCINTILLA.CPP_NUMBER,
        'Keyword'               : GTKSCINTILLA.CPP_KEYWORD,
        'String'                : GTKSCINTILLA.CPP_STRING,
        'Character'             : GTKSCINTILLA.CPP_CHARACTER,
        'UUID'                  : GTKSCINTILLA.CPP_UUID,
        'Preprocessor'          : GTKSCINTILLA.CPP_PREPROCESSOR,
        'Operator'              : GTKSCINTILLA.CPP_OPERATOR,
        'Identifier'            : GTKSCINTILLA.CPP_IDENTIFIER,
        'String not closed'     : GTKSCINTILLA.CPP_STRING_EOL,
        'Verbatim String (C#)'  : GTKSCINTILLA.CPP_STRING_VERBATIM,
        'Regular expression'    : GTKSCINTILLA.CPP_REGEX,
        'Comment line doc'      : GTKSCINTILLA.CPP_COMMENT_LINE_DOC,
        'Keyword 2'             : GTKSCINTILLA.CPP_KEYWORD2,
        'Comment doc keyword'   : GTKSCINTILLA.CPP_COMMENT_DOC_KEYWORD,
        'Comment doc error'     : GTKSCINTILLA.CPP_COMMENT_DOC_KEYWORD_ERROR },
    GTKSCINTILLA.LEXER_HTML : {
        'Text'                  : GTKSCINTILLA.HTML_TEXT,
        'Tag'                   : GTKSCINTILLA.HTML_TAG,
        'Tag unknown'           : GTKSCINTILLA.HTML_TAG_UNKNOWN,
        'Attribute'             : GTKSCINTILLA.HTML_ATTRIBUTE,
        'Attribute unknown'     : GTKSCINTILLA.HTML_ATTRIBUTE_UNKNOWN,
        'Number'                : GTKSCINTILLA.HTML_NUMBER,
        'String double quote'   : GTKSCINTILLA.HTML_DOUBLE_STRING,
        'String single quote'   : GTKSCINTILLA.HTML_SINGLE_STRING,
        'Other'                 : GTKSCINTILLA.HTML_OTHER,
        'Comment'               : GTKSCINTILLA.HTML_COMMENT,
        'Entity'                : GTKSCINTILLA.HTML_ENTITY,
        'XML tag ends (/>)'     : GTKSCINTILLA.HTML_TAG_END,
        'Identifier start (<?)' : GTKSCINTILLA.HTML_XML_START,
        'Identifier end (?>)'   : GTKSCINTILLA.HTML_XML_END,
        'Script'                : GTKSCINTILLA.HTML_SCRIPT,
        'ASP'                   : GTKSCINTILLA.HTML_ASP,
        'ASP Alt'               : GTKSCINTILLA.HTML_ASP_AT,
        'CDATA'                 : GTKSCINTILLA.HTML_CDATA,
        'Question'              : GTKSCINTILLA.HTML_QUESTION,
        'Value'                 : GTKSCINTILLA.HTML_VALUE,
        
        'XCode comment'         : GTKSCINTILLA.HTML_XC_COMMENT,
        
        'SGML default'           : GTKSCINTILLA.HTML_SGML_DEFAULT,
        'SGML command'           : GTKSCINTILLA.HTML_SGML_COMMAND,
        'SGML 1st param'         : GTKSCINTILLA.HTML_SGML_1ST_PARAM,
        'SGML string double'     : GTKSCINTILLA.HTML_SGML_DOUBLE_STRING,
        'SGML string single'     : GTKSCINTILLA.HTML_SGML_SIMPLE_STRING,
        'SGML error'             : GTKSCINTILLA.HTML_SGML_ERROR,
        'SGML special'           : GTKSCINTILLA.HTML_SGML_SPECIAL,
        'SGML entity'            : GTKSCINTILLA.HTML_SGML_ENTITY,
        'SGML comment'           : GTKSCINTILLA.HTML_SGML_COMMENT,
        'SGML 1st param comment' : GTKSCINTILLA.HTML_SGML_1ST_PARAM_COMMENT,
        'SGML block default'     : GTKSCINTILLA.HTML_SGML_BLOCK_DEFAULT,
        
        'JS Start'                  : GTKSCINTILLA.HTML_JS_START,
        'JS Default'                : GTKSCINTILLA.HTML_JS_DEFAULT,
        'JS Comment block'          : GTKSCINTILLA.HTML_JS_COMMENT_BLOCK,
        'JS Comment line'           : GTKSCINTILLA.HTML_JS_COMMENT_LINE,
        'JS Comment doc'            : GTKSCINTILLA.HTML_JS_COMMENT_DOC,
        'JS Number'                 : GTKSCINTILLA.HTML_JS_NUMBER,
        'JS Word'                   : GTKSCINTILLA.HTML_JS_WORD,
        'JS Keyword'                : GTKSCINTILLA.HTML_JS_KEYWORD,
        'JS String double quote'    : GTKSCINTILLA.HTML_JS_STRING_DOUBLE,
        'JS String single quote'    : GTKSCINTILLA.HTML_JS_STRING_SINGLE,
        'JS Symbol'                 : GTKSCINTILLA.HTML_JS_SYMBOL,
        'JS String EOL'             : GTKSCINTILLA.HTML_JS_EOL,
        
        'PY Start'                      : GTKSCINTILLA.HTML_PY_START,
        'PY Default'                    : GTKSCINTILLA.HTML_PY_DEFAULT,
        'PY Comment line'               : GTKSCINTILLA.HTML_PY_COMMENT_LINE,
        'PY Number'                     : GTKSCINTILLA.HTML_PY_NUMBER,
        'PY String double quote'        : GTKSCINTILLA.HTML_PY_STRING_DOUBLE,
        'PY String single quote'        : GTKSCINTILLA.HTML_PY_STRING_SINGLE,
        'PY Keyword'                    : GTKSCINTILLA.HTML_PY_KEYWORD,
        'PY Long string double quote'   : GTKSCINTILLA.HTML_PY_TRIPLE_SINGLE,
        'PY Long string single quote'   : GTKSCINTILLA.HTML_PY_TRIPLE_DOUBLE,
        'PY Class name definition'      : GTKSCINTILLA.HTML_PY_CLASS_STATEMENT,
        'PY Function name definition'   : GTKSCINTILLA.HTML_PY_DEF_STATEMENT,
        'PY Operator'                   : GTKSCINTILLA.HTML_PY_OPERATOR,
        'PY Identifier'                 : GTKSCINTILLA.HTML_PY_IDENTIFIER,
        
        'PHP Default'               : GTKSCINTILLA.HTML_PHP_DEFAULT,
        'PHP String double quote'   : GTKSCINTILLA.HTML_PHP_STRING_DOUBLE,
        'PHP String single quote'   : GTKSCINTILLA.HTML_PHP_STRING_SINGLE,
        'PHP Keyword'               : GTKSCINTILLA.HTML_PHP_KEYWORD,
        'PHP Number'                : GTKSCINTILLA.HTML_PHP_NUMBER,
        'PHP Variable'              : GTKSCINTILLA.HTML_PHP_VARIABLE,
        'PHP Comment block'         : GTKSCINTILLA.HTML_PHP_COMMENT_BLOCK,
        'PHP Comment line'          : GTKSCINTILLA.HTML_PHP_COMMENT_LINE,
        'PHP Variable in string'    : GTKSCINTILLA.HTML_PHP_STRING_VARIABLE,
        'PHP Operator'              : GTKSCINTILLA.HTML_PHP_OPERATOR },
    GTKSCINTILLA.LEXER_XML : {
        'Text'                  : GTKSCINTILLA.XML_DEFAULT,
        'Tag'                   : GTKSCINTILLA.XML_TAG,
        'Tag unknown'           : GTKSCINTILLA.XML_TAG_UNKNOWN,
        'Attribute'             : GTKSCINTILLA.XML_ATTRIBUTE,
        'Attribute unknown'     : GTKSCINTILLA.XML_ATTRIBUTE_UNKNOWN,
        'Number'                : GTKSCINTILLA.XML_NUMBER,
        'String double quote'   : GTKSCINTILLA.XML_DOUBLE_STRING,
        'String single quote'   : GTKSCINTILLA.XML_SINGLE_STRING,
        'Other'                 : GTKSCINTILLA.XML_OTHER,
        'Comment'               : GTKSCINTILLA.XML_COMMENT,
        'Entity'                : GTKSCINTILLA.XML_ENTITY,
        'Tag ends (/>)'         : GTKSCINTILLA.XML_TAG_END,
        'Identifier start (<?)' : GTKSCINTILLA.XML_START,
        'Identifier end (?>)'   : GTKSCINTILLA.XML_SCRIPT,
        'CDATA'                 : GTKSCINTILLA.XML_CDATA,
        'Question'              : GTKSCINTILLA.XML_QUESTION },
    GTKSCINTILLA.LEXER_PERL : {
        'White space'           : GTKSCINTILLA.PERL_WHITE_SPACE,
        'Error'                 : GTKSCINTILLA.PERL_ERROR,
        'Comment line'          : GTKSCINTILLA.PERL_COMMENT_LINE,
        'POD'                   : GTKSCINTILLA.PERL_POD,
        'Number'                : GTKSCINTILLA.PERL_NUMBER,
        'Keyword'               : GTKSCINTILLA.PERL_KEYWORD,
        'String'                : GTKSCINTILLA.PERL_STRING,
        'Character'             : GTKSCINTILLA.PERL_CHARACTER,
        'Punctuation'           : GTKSCINTILLA.PERL_PUNCTUATION,
        'Preprocessor'          : GTKSCINTILLA.PERL_PREPROCESSOR,
        'Operator'              : GTKSCINTILLA.PERL_OPERATOR,
        'Identifier'            : GTKSCINTILLA.PERL_IDENTIFIER,
        'Scalar'                : GTKSCINTILLA.PERL_SCALAR,
        'Array'                 : GTKSCINTILLA.PERL_ARRAY,
        'Hash'                  : GTKSCINTILLA.PERL_HASH,
        'Symbol table'          : GTKSCINTILLA.PERL_SYMBOL_TABLE,
        'Regular expression'    : GTKSCINTILLA.PERL_REGEX,
        'Regex substitution'    : GTKSCINTILLA.PERL_REGSUBST,
        'Long quote'            : GTKSCINTILLA.PERL_LONG_QUOTE,
        'Backticks'             : GTKSCINTILLA.PERL_BACKTICKS,
        'Data section'          : GTKSCINTILLA.PERL_DATA_SECTION,
        'Here doc delimiter'    : GTKSCINTILLA.PERL_HERE_DELIM,
        'Here doc Q'            : GTKSCINTILLA.PERL_HERE_Q,
        'Here doc QQ'           : GTKSCINTILLA.PERL_HERE_QQ,
        'Here doc QX'           : GTKSCINTILLA.PERL_HERE_QX,
        'String Q'              : GTKSCINTILLA.PERL_STRING_Q,
        'String QQ'             : GTKSCINTILLA.PERL_STRING_QQ,
        'String QX'             : GTKSCINTILLA.PERL_STRING_QX,
        'String QR'             : GTKSCINTILLA.PERL_STRING_QR,
        'String QW'             : GTKSCINTILLA.PERL_STRING_QW },
    GTKSCINTILLA.LEXER_LATEX : {
        'White space'   : GTKSCINTILLA.LATEX_WHITE_SPACE,
        'Command'       : GTKSCINTILLA.LATEX_COMMAND,
        'Tag'           : GTKSCINTILLA.LATEX_TAG,
        'Math'          : GTKSCINTILLA.LATEX_MATH,
        'Comment'       : GTKSCINTILLA.LATEX_COMMENT },
    GTKSCINTILLA.LEXER_LUA : {
        'White space'       : GTKSCINTILLA.LUA_WHITE_SPACE,
        'Comment block'     : GTKSCINTILLA.LUA_COMMENT_BLOCK,
        'Comment line'      : GTKSCINTILLA.LUA_COMMENT_LINE,
        'Comment doc'       : GTKSCINTILLA.LUA_COMMENT_DOC,
        'Number'            : GTKSCINTILLA.LUA_NUMBER,
        'Keyword'           : GTKSCINTILLA.LUA_KEYWORD,
        'String'            : GTKSCINTILLA.LUA_STRING,
        'Character'         : GTKSCINTILLA.LUA_CHARACTER,
        'String literal'    : GTKSCINTILLA.LUA_LITERAL_STRING,
        'Preprocessor'      : GTKSCINTILLA.LUA_PREPROCESSOR,
        'Operator'          : GTKSCINTILLA.LUA_OPERATOR,
        'Identifier'        : GTKSCINTILLA.LUA_IDENTIFIER,
        'String EOL'        : GTKSCINTILLA.LUA_STRING_EOL },
    GTKSCINTILLA.LEXER_ERRORLIST : {
        'White space'   : GTKSCINTILLA.ERROR_DEFAULT,
        'Python'        : GTKSCINTILLA.ERROR_PYTHON,
        'GCC'           : GTKSCINTILLA.ERROR_GCC,
        'Microsoft'     : GTKSCINTILLA.ERROR_MS,
        'Command'       : GTKSCINTILLA.ERROR_CMD,
        'Borland'       : GTKSCINTILLA.ERROR_BORLAND,
        'Perl'          : GTKSCINTILLA.ERROR_PERL,
        'Diff changed'  : GTKSCINTILLA.ERROR_DIFF_CHANGED,
        'Diff added'    : GTKSCINTILLA.ERROR_DIFF_ADDITION,
        'Diff removed'  : GTKSCINTILLA.ERROR_DIFF_DELETION,
        'Diff message'  : GTKSCINTILLA.ERROR_DIFF_MESSAGE },
    GTKSCINTILLA.LEXER_BATCH : {
        'White space'   : GTKSCINTILLA.BATCH_DEFAULT,
        'Comment'       : GTKSCINTILLA.BATCH_COMMENT,
        'Keyword'       : GTKSCINTILLA.BATCH_WORD,
        'Label'         : GTKSCINTILLA.BATCH_LABEL,
        'Hide (@)'      : GTKSCINTILLA.BATCH_HIDE,
        'Command'       : GTKSCINTILLA.BATCH_COMMAND,
        'Variable'      : GTKSCINTILLA.BATCH_IDENTIFIER,
        'Operator'      : GTKSCINTILLA.BATCH_OPERATOR },
    GTKSCINTILLA.LEXER_MAKEFILE : {
        'Default'       : GTKSCINTILLA.MAKEFILE_WHITE_SPACE,
        'Comment'       : GTKSCINTILLA.MAKEFILE_COMMENT,
        'Preprocessor'  : GTKSCINTILLA.MAKEFILE_PREPROCESSOR,
        'Variable'      : GTKSCINTILLA.MAKEFILE_IDENTIFIER,
        'Operator'      : GTKSCINTILLA.MAKEFILE_OPERATOR,
        'Error'         : GTKSCINTILLA.MAKEFILE_IDEOL },
    GTKSCINTILLA.LEXER_CONF : {
        'White space'       : GTKSCINTILLA.CONF_DEFAULT,
        'Comment'           : GTKSCINTILLA.CONF_COMMENT,
        'Number'            : GTKSCINTILLA.CONF_NUMBER,
        'Identifier'        : GTKSCINTILLA.CONF_IDENTIFIER,
        'File extension'    : GTKSCINTILLA.CONF_EXTENSION,
        'Parameter'         : GTKSCINTILLA.CONF_PARAMETER,
        'String'            : GTKSCINTILLA.CONF_STRING,
        'Operator'          : GTKSCINTILLA.CONF_OPERATOR,
        'IP address'        : GTKSCINTILLA.CONF_IP,
        'Directive'         : GTKSCINTILLA.CONF_DIRECTIVE },
    GTKSCINTILLA.LEXER_AVE : {
        'White space'   : GTKSCINTILLA.AVE_DEFAULT,
        'Comment'       : GTKSCINTILLA.AVE_COMMENT,
        'Number'        : GTKSCINTILLA.AVE_NUMBER,
        'Word'          : GTKSCINTILLA.AVE_WORD,
        'Keyword'       : GTKSCINTILLA.AVE_KEYWORD,
        'Statement'     : GTKSCINTILLA.AVE_STATEMENT,
        'String'        : GTKSCINTILLA.AVE_STRING,
        'Enum'          : GTKSCINTILLA.AVE_ENUM,
        'String EOL'    : GTKSCINTILLA.AVE_STRINGEOL,
        'Identifier'    : GTKSCINTILLA.AVE_IDENTIFIER,
        'Operator'      : GTKSCINTILLA.AVE_OPERATOR },
    GTKSCINTILLA.LEXER_ADA : {
        'White space'   : GTKSCINTILLA.ADA_DEFAULT,
        'Comment'       : GTKSCINTILLA.ADA_COMMENT,
        'Number'        : GTKSCINTILLA.ADA_NUMBER,
        'Keyword'       : GTKSCINTILLA.ADA_WORD,
        'String'        : GTKSCINTILLA.ADA_STRING,
        'Character'     : GTKSCINTILLA.ADA_CHARACTER,
        'Operator'      : GTKSCINTILLA.ADA_OPERATOR,
        'Identifier'    : GTKSCINTILLA.ADA_IDENTIFIER,
        'String EOL'    : GTKSCINTILLA.ADA_STRINGEOL },
    GTKSCINTILLA.LEXER_BAAN : {
    },
    GTKSCINTILLA.LEXER_LISP : {
        'Default'       : GTKSCINTILLA.LISP_DEFAULT,
        'Comment'       : GTKSCINTILLA.LISP_COMMENT,
        'Number'        : GTKSCINTILLA.LISP_NUMBER,
        'Keyword'       : GTKSCINTILLA.LISP_KEYWORD,
        'String'        : GTKSCINTILLA.LISP_STRING,
        'String EOL'    : GTKSCINTILLA.LISP_STRING_EOL,
        'Identifier'    : GTKSCINTILLA.LISP_IDENTIFIER,
        'Operator'      : GTKSCINTILLA.LISP_OPERATOR },
    GTKSCINTILLA.LEXER_RUBY : {
        'White space'               : GTKSCINTILLA.RUBY_WHITE_SPACE,
        'Comment line'              : GTKSCINTILLA.RUBY_COMMENT_LINE,
        'Number'                    : GTKSCINTILLA.RUBY_NUMBER,
        'String double quote'       : GTKSCINTILLA.RUBY_STRING_DOUBLE,
        'String single quote'       : GTKSCINTILLA.RUBY_STRING_SINGLE,
        'Keyword'                   : GTKSCINTILLA.RUBY_KEYWORD,
        'Long string double quote'  : GTKSCINTILLA.RUBY_TRIPLE_DOUBLE,
        'Long string single quote'  : GTKSCINTILLA.RUBY_TRIPLE_SINGLE,
        'Class name definition'     : GTKSCINTILLA.RUBY_CLASS_STATEMENT,
        'Function name definition'  : GTKSCINTILLA.RUBY_DEF_STATEMENT,
        'Operator'                  : GTKSCINTILLA.RUBY_OPERATOR,
        'Identifier'                : GTKSCINTILLA.RUBY_IDENTIFIER,
        'Comment block'             : GTKSCINTILLA.RUBY_COMMENT_BLOCK,
        'String EOL'                : GTKSCINTILLA.RUBY_STRING_EOL },
    GTKSCINTILLA.LEXER_EIFFEL : {
    },
    GTKSCINTILLA.LEXER_EIFFELKW : {
    },
    GTKSCINTILLA.LEXER_NNCRONTAB : {
    },
    GTKSCINTILLA.LEXER_SQL : {
        'White space'           : GTKSCINTILLA.SQL_WHITE_SPACE,
        'Comment block'         : GTKSCINTILLA.SQL_COMMENT_BLOCK,
        'Comment line'          : GTKSCINTILLA.SQL_COMMENT_LINE,
        'Comment doc'           : GTKSCINTILLA.SQL_COMMENT_DOC,
        'Number'                : GTKSCINTILLA.SQL_NUMBER,
        'Keyword'               : GTKSCINTILLA.SQL_KEYWORD,
        'String double quote'   : GTKSCINTILLA.SQL_DOUBLE_STRING,
        'String single quote'   : GTKSCINTILLA.SQL_SINGLE_STRING,
        'Symbols'               : GTKSCINTILLA.SQL_SYMBOLS,
        'Preprocessor'          : GTKSCINTILLA.SQL_PREPROCESSOR,
        'Operator'              : GTKSCINTILLA.SQL_OPERATOR,
        'Identifier'            : GTKSCINTILLA.SQL_IDENTIFIER,
        'String EOL'            : GTKSCINTILLA.SQL_STRING_EOL },
    GTKSCINTILLA.LEXER_VB : {
        'White space'       : GTKSCINTILLA.VB_WHITE_SPACE,
        'Comment block'     : GTKSCINTILLA.VB_COMMENT_BLOCK,
        'Comment line'      : GTKSCINTILLA.VB_COMMENT_LINE,
        'Comment doc'       : GTKSCINTILLA.VB_COMMENT_DOC,
        'Number'            : GTKSCINTILLA.VB_NUMBER,
        'Keyword'           : GTKSCINTILLA.VB_WORD,
        'String'            : GTKSCINTILLA.VB_STRING,
        'Character'         : GTKSCINTILLA.VB_CHARACTER,
        'UUID'              : GTKSCINTILLA.VB_UUID,
        'Preprocessor'      : GTKSCINTILLA.VB_PREPROCESSOR,
        'Operator'          : GTKSCINTILLA.VB_OPERATOR,
        'Identifier'        : GTKSCINTILLA.VB_IDENTIFIER,
        'String EOL'        : GTKSCINTILLA.VB_STRING_EOL },
    GTKSCINTILLA.LEXER_PROPERTIES : {
        'White space'       : GTKSCINTILLA.PROPERTIES_DEFAULT,
        'Comment'           : GTKSCINTILLA.PROPERTIES_COMMENT,
        'Section'           : GTKSCINTILLA.PROPERTIES_SECTION,
        'Assignment'        : GTKSCINTILLA.PROPERTIES_OPERATOR,
        'Default value (@)' : GTKSCINTILLA.PROPERTIES_DEFAULT_VAL
        },
    GTKSCINTILLA.LEXER_DIFF : {
        'Context lines' : GTKSCINTILLA.DIFF_DEFAULT,
        'Comment'       : GTKSCINTILLA.DIFF_COMMENT,
        'Command'       : GTKSCINTILLA.DIFF_COMMAND,
        'File name'     : GTKSCINTILLA.DIFF_FILES,
        'Position'      : GTKSCINTILLA.DIFF_POSITION,
        'Line removed'  : GTKSCINTILLA.DIFF_DEL_LINE,
        'Line added'    : GTKSCINTILLA.DIFF_ADD_LINE },
    GTKSCINTILLA.LEXER_PASCAL : {
        'White space'           : GTKSCINTILLA.PASCAL_WHITE_SPACE,
        'Comment block'         : GTKSCINTILLA.PASCAL_COMMENT_BLOCK,
        'Comment line'          : GTKSCINTILLA.PASCAL_COMMENT_LINE,
        'Comment doc'           : GTKSCINTILLA.PASCAL_COMMENT_DOC,
        'Number'                : GTKSCINTILLA.PASCAL_NUMBER,
        'Keyword'               : GTKSCINTILLA.PASCAL_KEYWORD,
        'String double quote'   : GTKSCINTILLA.PASCAL_DOUBLE_STRING,
        'String single quote'   : GTKSCINTILLA.PASCAL_SINGLE_STRING,
        'Symbols'               : GTKSCINTILLA.PASCAL_SYMBOLS,
        'Preprocessor'          : GTKSCINTILLA.PASCAL_PREPROCESSOR },
}

def _merge_font(font):
    """Merge the given X font descripton with the deafult font"""
    df = string.split(Moleskine.app.prefs['default style/font'], '-')
    sf = string.split(font, '-')
    for i in range(min(len(df), len(sf))):
        if sf[i] == '*':
            sf[i] = df[i]
    return string.join(sf, '-')

def _split_font(font):
    """Split the given X font descripton from the deafult font
    
    If the given font is equivalent to the default, the function returns None.
    """
    df = string.split(Moleskine.app.prefs['default style/font'], '-')
    sf = string.split(font, '-')
    count = 0
    for i in range(min(len(df), len(sf))):
        if sf[i] == df[i]:
            sf[i] = '*'
            count = count + 1
    if count == min(len(df), len(sf)):
        return None
    else:
        return string.join(sf, '-')[1:]

class Language:
    def __init__(self, name, lexer):
        self.name = name
        if lexer is None:
            self.load()
        else:
            self.lexer = lexer
            self.init_lang()
            self.init_styles()
    
    def init_lang(self):
        self.keywords = ['', '', '', '', '']
        self.tab_size = 4
        self.soft_tabs = 1
        self.indent_size = 4
        self.show_indent = 1
        self.auto_indent = 1
        self.edge_indicator = 0
        self.edge_column = 80
        self.edge_color = 0
        self.rules = []

    def init_styles(self):
        self.styles = {}
        if not Styles.has_key(self.lexer):
            self.styles[0] = {
                'font': None,
                'fore': None,
                'back': None }
            return
        for i in Styles[self.lexer].values():
            self.styles[i] = {
                'font': None,
                'fore': None,
                'back': None }
    
    def load(self):
        gnome.config.push_prefix(os.path.join(_CONFIG_FILE, self.name) + '/')
        
        self.lexer = gnome.config.get_int('lexer')
        self.keywords = [None, None, None, None, None]
        self.keywords[0] = gnome.config.get_string('keywords1')
        self.keywords[1] = gnome.config.get_string('keywords2')
        self.keywords[2] = gnome.config.get_string('keywords3')
        self.keywords[3] = gnome.config.get_string('keywords4')
        self.keywords[4] = gnome.config.get_string('keywords5')
        self.tab_size = gnome.config.get_int('tab_size')
        self.soft_tabs = gnome.config.get_bool('soft_tabs')
        self.indent_size = gnome.config.get_int('indent_size')
        self.show_indent = gnome.config.get_bool('show_indent')
        self.auto_indent = gnome.config.get_bool('auto_indent')
        self.edge_indicator = gnome.config.get_int('edge_indicator')
        self.edge_column = gnome.config.get_int('edge_column')
        self.edge_color = gnome.config.get_int('edge_color')
        
        self.load_styles()
        self.load_rules()
        
        gnome.config.pop_prefix()
    
    def load_rules(self):
        self.rules = []
        rules_count = gnome.config.get_int('rules_count')
        for i in range(rules_count):
            type = gnome.config.get_int('type_%i_type' % i)
            rule = gnome.config.get_string('type_%i_rule' % i)
            self.rules.append((type, rule))
    
    def load_styles(self):
        self.init_styles()
        
        if not Styles.has_key(self.lexer):
            return
        
        for style in Styles[self.lexer].values():
            s = self.styles[style]
            s['font'] = gnome.config.get_string('style_%i_font' % style)
            s['fore'] = gnome.config.get_string('style_%i_fore' % style)
            s['back'] = gnome.config.get_string('style_%i_back' % style)
    
    def save(self):
        gnome.config.push_prefix(os.path.join(_CONFIG_FILE, self.name) + '/')
        
        gnome.config.set_int('lexer', self.lexer)
        if self.keywords[0] is not None:
            gnome.config.set_string('keywords1', self.keywords[0])
        if self.keywords[1] is not None:
            gnome.config.set_string('keywords2', self.keywords[1])
        if self.keywords[2] is not None:
            gnome.config.set_string('keywords3', self.keywords[2])
        if self.keywords[3] is not None:
            gnome.config.set_string('keywords4', self.keywords[3])
        if self.keywords[4] is not None:
            gnome.config.set_string('keywords5', self.keywords[4])
        gnome.config.set_int('tab_size', self.tab_size)
        gnome.config.set_bool('soft_tabs', self.soft_tabs)
        gnome.config.set_int('indent_size', self.indent_size)
        gnome.config.set_bool('show_indent', self.show_indent)
        gnome.config.set_bool('auto_indent', self.auto_indent)
        gnome.config.set_int('edge_indicator', self.edge_indicator)
        gnome.config.set_int('edge_column', self.edge_column)
        gnome.config.set_int('edge_color', self.edge_color)
        
        self.save_styles()
        self.save_rules()
        
        gnome.config.pop_prefix()
    
    def save_rules(self):
        gnome.config.set_int('rules_count', len(self.rules))
        for i in range(len(self.rules)):
            gnome.config.set_int('type_%i_type' % i, self.rules[i][0])
            gnome.config.set_string('type_%i_rule' % i, self.rules[i][1])
    
    def save_styles(self):
        if not Styles.has_key(self.lexer):
            return
        
        for style in self.styles.keys():
            s = self.styles[style]
            if s['font'] is not None:
                if Moleskine.app.prefs['default style/font'] != s['font']:
                    gnome.config.set_string('style_%i_font' % style, s['font'])
                else:
                    s['font'] = None
            if s['fore'] is not None:
                if Moleskine.app.prefs['default style/fore'] != s['fore']:
                    gnome.config.set_string('style_%i_fore' % style, s['fore'])
                else:
                    s['fore'] = None
            if s['back'] is not None:
                if Moleskine.app.prefs['default style/back'] != s['back']:
                    gnome.config.set_string('style_%i_back' % style, s['back'])
                else:
                    s['back'] = None

class LanguagesList(UserDict):
    def new(self, name, lexer = None):
        self.data[name] = Language(name, lexer)
    
    def remove(self, name):
        del self.data[name]
    
    def load(self):
        gnome.config.push_prefix(_CONFIG_FILE)
        stock_langs = gnome.config.get_string('Languages/stock_languages')
        langs = gnome.config.get_string('Languages/languages')
        gnome.config.pop_prefix()
        
        langs_list = []
        if langs is not None:
            langs_list = string.split(langs, '\n')
        stock_langs_list = []
        if stock_langs is not None:
            stock_langs_list = string.split(stock_langs, '\n')
        
        d = {}
        for i in langs_list:
            d[i] = None
        for i in stock_langs_list:
            d[i] = None
        langs_list = d.keys()
        
        for lang_name in langs_list:
            self.new(lang_name)
        
        gnome.config.drop_all()
    
    def save(self):
        gnome.config.clean_file(_CONFIG_FILE)
        gnome.config.push_prefix(_CONFIG_FILE)
        langs = string.join(self.keys(), '\n')
        gnome.config.set_string('Languages/languages', langs)
        gnome.config.pop_prefix()
        
        for lang in self.values():
            lang.save()
        
        gnome.config.sync()
        gnome.config.drop_all()
