# actions.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import Moleskine
import GTKSCINTILLA

me = Moleskine.app

class FindFlags:
    match_case = 0
    whole_words = 0
    word_start = 0
    regexp = 0
    backward = 0
    text = ''

def search_text(text=FindFlags.text,
                match_case=FindFlags.match_case,
                whole_words=FindFlags.whole_words,
                word_start=FindFlags.word_start,
                regexp=FindFlags.regexp,
                backward=FindFlags.backward):
    """Search for a string in the active document
    
    The function starts searching from the current position. If the word is
    found the function selects the string and returns a tuple containing its
    starting and ending positions, or None if the string hasn't been found.
    """
    FindFlags.text = text
    FindFlags.match_case = match_case
    FindFlags.whole_words = whole_words
    FindFlags.word_start = word_start
    FindFlags.regexp = regexp
    FindFlags.backward = backward
    
    doc = me.get_current_document()
    flags = backward * GTKSCINTILLA.FIND_DOWN + \
            match_case * GTKSCINTILLA.FIND_MATCH_CASE + \
            whole_words * GTKSCINTILLA.FIND_WHOLE_WORDS + \
            word_start * GTKSCINTILLA.FIND_WORD_START
    if backward:
        start = doc.get_selection_start() - 1
        end = 0
    else:
        start = doc.get_selection_end()
        end = doc.get_length()
    result = doc.find_text(flags, text, (start, end))
    if result:
        doc.set_selection_start(result[0])
        doc.set_selection_end(result[1])
    else:
        doc.set_selection_end(doc.get_selection_start())
    return result

def search_next():
    """Search for the next string matching the last used criterion
    
    The function uses the same options and the same criterion used in the
    last search.
    """
    return search_text(text=FindFlags.text,
                       match_case=FindFlags.match_case,
                       whole_words=FindFlags.whole_words,
                       word_start=FindFlags.word_start,
                       regexp=FindFlags.regexp,
                       backward=0)

def search_previous():
    """Search for the previous string matching the last used criterion
    
    The function uses the same options and the same criterion used in the
    last search.
    """
    return search_text(text=FindFlags.text,
                       match_case=FindFlags.match_case,
                       whole_words=FindFlags.whole_words,
                       word_start=FindFlags.word_start,
                       regexp=FindFlags.regexp,
                       backward=1)
